@@ -30,9 +30,9 @@
 static int ratedPositions, wonPositions, searchCalled, moveCount;
 static int normalCount, pushCount, outCount, cutoffCount;
 
-/*********************** Class MainCombination *************************/
+/*********************** Class PrincipalVariation *************************/
 
-void MainCombination::clear(int d)
+void PrincipalVariation::clear(int d)
 {
 	int i,j;
 	
@@ -43,7 +43,7 @@ void MainCombination::clear(int d)
 	actMaxDepth = (d<maxDepth) ? d:maxDepth-1;
 }
 
-void MainCombination::update(int d, Move& m)
+void PrincipalVariation::update(int d, Move& m)
 {
 	int i;
 
@@ -98,20 +98,22 @@ int Board::stoneValue[]= { 0,-800,-1800,-3000,-4400,-6000 };
 
 int Board::direction[]= { -11,1,12,11,-1,-12,-11,1 };
 
-/* Values for possible moves: 
- * (move2, move3, push1, push2, out) */
-int Board::moveValue[]= {
-  1,3,5,7,30 };
+/* Default Values for moves values (see Move.h) */
+int Board::moveValue[]= { 40,30,30, 15,14,13, 5,5,5, 2,2,2, 1 };
+
+/* Default Values for connect values (see Move.h) */
+int Board::connectValue[]= { 2, 5, 4, 3 };
+
 
 Board::Board()
 {
-	clear();
-	breakOut = bUpdateSpy = false;
-	boardOk = true;
-	spyLevel = 0;
-	debug = 0;
-	realMaxDepth = 1;
-	setFieldValues();
+  clear();
+  breakOut = bUpdateSpy = false;
+  spyLevel = 0;
+  spyDepth = 0;
+  debug = 0;
+  realMaxDepth = 1;
+  setFieldValues();
 }
 
 void Board::setFieldValues()
@@ -144,10 +146,15 @@ void Board::readRating(KConfig *config)
   }
 
   if (config->readListEntry("MoveValues", list)) {
-    for(int i=0;i<5;i++)
+    for(int i=0;i<Move::typeCount;i++)
       moveValue[i] = (tmp = list.at(i)).toInt();
   }
 
+  if (config->readListEntry("ConnectValues", list)) {
+    for(int i=0;i<ConnectCounter::connectCount;i++)
+      connectValue[i] = (tmp = list.at(i)).toInt();
+  }
+
   if (config->readListEntry("RingValues", list)) {
     for(int i=0;i<5;i++)
       ringValue[i] = (tmp = list.at(i)).toInt();
@@ -169,10 +176,16 @@ void Board::saveRating(KConfig *config)
 		stoneValue[3], stoneValue[4], stoneValue[5]);
   config->writeEntry("StoneValues", entry);
 
-  entry.sprintf("%d,%d,%d,%d,%d", moveValue[0], moveValue[1],
-		moveValue[2], moveValue[3], moveValue[4]);
+  entry.sprintf("%d", moveValue[0]);
+  for(int i=1;i<Move::typeCount;i++)
+    entry += QString(", %1").arg( moveValue[i] );
   config->writeEntry("MoveValues", entry);
 
+  entry.sprintf("%d", connectValue[0]);
+  for(int i=1;i<ConnectCounter::connectCount;i++)
+    entry += QString(", %1").arg( connectValue[i] );
+  config->writeEntry("ConnectValues", entry);
+
   entry.sprintf("%d,%d,%d,%d,%d", ringValue[0], ringValue[1],
 		ringValue[2], ringValue[3], ringValue[4]);
   config->writeEntry("RingValues", entry);
@@ -193,7 +206,6 @@ void Board::begin(int startColor)
 	storedFirst = storedLast = 0;
 	color = startColor;
 	color1Count = color2Count = 14;
-	boardOk = true;
         random.setSeed(0); // Initialize random sequence
 }
 
@@ -205,127 +217,127 @@ void Board::clear()
 	  field[i] = (startBoard[i] == out) ? out: free;
 	storedFirst = storedLast = 0;
 	color1Count = color2Count = 0;
-	boardOk = true;
 }
 
 /* generate moves starting at field <startField> */
 void Board::generateFieldMoves(int startField, MoveList& list)
 {
-	int d, dir, c, actField, left, right;
-	int opponent = (color == color1) ? color2 : color1;
+  int d, dir, c, actField;
+  bool left, right;
+  int opponent = (color == color1) ? color2 : color1;
 
-	ASSERT( field[startField] == color );
-	
-	/* 6 directions	*/
-	for(d=1;d<7;d++) {
-		dir = direction[d];
-		  
-		/* 2nd field */
-		c = field[actField = startField+dir];
-		if (c == free) {			
-			/* (c .) */
-			list.insert(startField, d, Move::move1);
-			continue;
-		}		
-		if (c != color) 
-		  continue;
-		
-		/* 2nd == color */
-		
-		left = (field[startField+direction[d-1]] == free);
-		if (left) {
-			left = (field[actField+direction[d-1]] == free);
-			if (left) 
-			  /* 2 left */
-			  list.insert(startField, d, Move::left2);
-		}
-		
-		right = (field[startField+direction[d+1]] == free);
-		if (right) {
-			right = (field[actField+direction[d+1]] == free);
-			if (right) 
-			  /* 2 right */
-			  list.insert(startField, d, Move::right2);
-		}
-		
-		/* 3rd field */
-		c = field[actField += dir];
-		if (c == free) {
-			/* (c c .) */
-			list.insert(startField, d, Move::move2);
-			continue;
-		}		
-		else if (c == opponent) {
-			
-			/* 4th field */
-			c = field[actField += dir];
-			if (c == free) {
-				/* (c c o .) */
-				list.insert(startField, d, Move::push1with2);
-			}
-			else if (c == out) {
-				/* (c c o |) */
-				list.insert(startField, d, Move::out1with2);
-			}
-			continue;
-		}
-		if (c != color)
-		  continue;
-		
-		/* 3nd == color */
+  ASSERT( field[startField] == color );
+  
+  /* 6 directions	*/
+  for(d=1;d<7;d++) {
+    dir = direction[d];
+    
+    /* 2nd field */
+    c = field[actField = startField+dir];
+    if (c == free) {			
+      /* (c .) */
+      list.insert(startField, d, Move::move1);
+      continue;
+    }		
+    if (c != color) 
+      continue;
+    
+    /* 2nd == color */
+    
+    left = (field[startField+direction[d-1]] == free);
+    if (left) {
+      left = (field[actField+direction[d-1]] == free);
+      if (left) 
+	/* 2 left */
+	list.insert(startField, d, Move::left2);
+    }
+    
+    right = (field[startField+direction[d+1]] == free);
+    if (right) {
+      right = (field[actField+direction[d+1]] == free);
+      if (right) 
+	/* 2 right */
+	list.insert(startField, d, Move::right2);
+    }
+    
+    /* 3rd field */
+    c = field[actField += dir];
+    if (c == free) {
+      /* (c c .) */
+      list.insert(startField, d, Move::move2);
+      continue;
+    }		
+    else if (c == opponent) {
+      
+      /* 4th field */
+      c = field[actField += dir];
+      if (c == free) {
+	/* (c c o .) */
+	list.insert(startField, d, Move::push1with2);
+      }
+      else if (c == out) {
+	/* (c c o |) */
+	list.insert(startField, d, Move::out1with2);
+      }
+      continue;
+    }
+    if (c != color)
+      continue;
+    
+    /* 3nd == color */
 
-		if (left) {
-			if (field[actField+direction[d-1]] == free) 
-			  /* 3 left */
-			  list.insert(startField, d, Move::left3);
-		}
-
-		if (right) {
-			if (field[actField+direction[d+1]] == free)
-			  /* 3 right */
-			  list.insert(startField, d, Move::right3);
-		}
-		
-		/* 4th field */
-		c = field[actField += dir];
-		if (c == free) {
-			/* (c c c .) */
-			list.insert(startField, d, Move::move3);
-			continue;
-		}
-		if (c != opponent)
-		  continue;
-
-		/* 4nd == opponent */
-		
-		/* 5. field */
-		c = field[actField += dir];
-		if (c == free) {
-			/* (c c c o .) */
-			list.insert(startField, d, Move::push1with3);
-			continue;
-		}
-		else if (c == out) {
-			/* (c c c o |) */
-			list.insert(startField, d, Move::out1with3);
-			continue;
-		}
-		if (c != opponent)
-		  continue;
-
-		/* 5nd == opponent */
-
-		/* 6. field */
-		c = field[actField += dir];
-		if (c == free) {
-			/* (c c c o o .) */
-			list.insert(startField, d, Move::push2);
-		}
-		else if (c == out) {
-			/* (c c c o o |) */
-			list.insert(startField, d, Move::out2);
-		}
-	}
+    if (left) {
+      if (field[actField+direction[d-1]] == free) 
+	/* 3 left */
+	list.insert(startField, d, Move::left3);
+    }
+    
+    if (right) {
+      if (field[actField+direction[d+1]] == free)
+	/* 3 right */
+	list.insert(startField, d, Move::right3);
+    }
+    
+    /* 4th field */
+    c = field[actField += dir];
+    if (c == free) {
+      /* (c c c .) */
+      list.insert(startField, d, Move::move3);
+      continue;
+    }
+    if (c != opponent)
+      continue;
+    
+    /* 4nd == opponent */
+    
+    /* 5. field */
+    c = field[actField += dir];
+    if (c == free) {
+      /* (c c c o .) */
+      list.insert(startField, d, Move::push1with3);
+      continue;
+    }
+    else if (c == out) {
+      /* (c c c o |) */
+      list.insert(startField, d, Move::out1with3);
+      continue;
+    }
+    if (c != opponent)
+      continue;
+    
+    /* 5nd == opponent */
+    
+    /* 6. field */
+    c = field[actField += dir];
+    if (c == free) {
+      /* (c c c o o .) */
+      list.insert(startField, d, Move::push2);
+    }
+    else if (c == out) {
+      /* (c c c o o |) */
+      list.insert(startField, d, Move::out2);
+    }
+  }
 }
 
 
@@ -333,8 +345,6 @@ void Board::generateMoves(MoveList& list)
 {
 	int actField, f;
 
-	ASSERT( boardOk );
-
 	for(f=0;f<RealFields;f++) {
 		actField = order[f];
 		if ( field[actField] == color)
@@ -349,7 +359,6 @@ void Board::playMove(const Move& m)
 	int f, dir, dir2;
 	int opponent = (color == color1) ? color2:color1;
 
-	CHECK( boardOk );
 	CHECK( isConsistent() );
 	
 	if (++storedLast == MvsStored) storedLast = 0;
@@ -489,144 +498,145 @@ void Board::playMove(const Move& m)
 	
 }
 
-void Board::takeBack()
+bool Board::takeBack()
 {
-	int f, dir, dir2;
-	int opponent = color;
-	Move& m = storedMove[storedLast];
-
-	CHECK( boardOk );
-	CHECK( isConsistent() );
-	
-	if (storedFirst == storedLast) return;
-
-	/* change actual color */
-	color = (color == color1) ? color2:color1;	
-
-	if (m.isOutMove()) {
-		if (color == color1)
-		  color2Count++;
-		else
-		  color1Count++;
-	}       
-	
-	f = m.field;
-	CHECK( field[f] == free );
-	field[f] = color;
-	dir = direction[m.direction];
-	
-	switch(m.type) {
-	 case Move::out2:        /* (. c c c o |) => (c c c o o |) */
-		CHECK( field[f + dir] == color );
-		CHECK( field[f + 2*dir] == color );
-		CHECK( field[f + 3*dir] == color );
-		CHECK( field[f + 4*dir] == opponent );
-		CHECK( field[f + 5*dir] == out );
-		field[f + 3*dir] = opponent;
-		break;
-	 case Move::out1with3:   /* (. c c c |) => (c c c o |) */
-		CHECK( field[f + dir] == color );
-		CHECK( field[f + 2*dir] == color );
-		CHECK( field[f + 3*dir] == color );
-		CHECK( field[f + 4*dir] == out );
-		field[f + 3*dir] = opponent;
-		break;
-	 case Move::move3:       /* (. c c c) => (c c c .)     */
-		CHECK( field[f + dir] == color );
-		CHECK( field[f + 2*dir] == color );
-		CHECK( field[f + 3*dir] == color );
-		field[f + 3*dir] = free;
-		break;
-	 case Move::out1with2:   /* (. c c | ) => (c c o |)     */
-		CHECK( field[f + dir] == color );
-		CHECK( field[f + 2*dir] == color );
-		CHECK( field[f + 3*dir] == out );
-		field[f + 2*dir] = opponent;
-		break;
-	 case Move::move2:       /* (. c c) => (c c .)       */
-		CHECK( field[f + dir] == color );
-		CHECK( field[f + 2*dir] == color );
-		field[f + 2*dir] = free;
-		break;
-	 case Move::push2:       /* (. c c c o o) => (c c c o o .) */
-		CHECK( field[f + dir] == color );
-		CHECK( field[f + 2*dir] == color );
-		CHECK( field[f + 3*dir] == color );
-		CHECK( field[f + 4*dir] == opponent );
-		CHECK( field[f + 5*dir] == opponent );
-		field[f + 3*dir] = opponent;
-		field[f + 5*dir] = free;
-		break;
-	 case Move::left3:
-		dir2 = direction[m.direction-1];
-		CHECK( field[f + dir] == free );
-		CHECK( field[f + 2*dir] == free );
-		CHECK( field[f + dir2] == color );
-		CHECK( field[f + dir+dir2] == color );
-		CHECK( field[f + 2*dir+dir2] == color );
-		field[f+dir2] = free;
-		field[f+=dir] = color;
-		field[f+dir2] = free;
-		field[f+=dir] = color;
-		field[f+dir2] = free;
-		break;
-	 case Move::right3:
-		dir2 = direction[m.direction+1];
-		CHECK( field[f + dir] == free );
-		CHECK( field[f + 2*dir] == free );
-		CHECK( field[f + dir2] == color );
-		CHECK( field[f + dir+dir2] == color );
-		CHECK( field[f + 2*dir+dir2] == color );
-		field[f+dir2] = free;
-		field[f+=dir] = color;
-		field[f+dir2] = free;
-		field[f+=dir] = color;
-		field[f+dir2] = free;
-		break;
-	 case Move::push1with3:   /* (. c c c o) => (c c c o .) */
-		CHECK( field[f + dir] == color );
-		CHECK( field[f + 2*dir] == color );
-		CHECK( field[f + 3*dir] == color );
-		CHECK( field[f + 4*dir] == opponent );
-		field[f + 3*dir] = opponent;
-		field[f + 4*dir] = free;
-		break;
-	 case Move::push1with2:   /* (. c c o) => (c c o .) */
-		CHECK( field[f + dir] == color );
-		CHECK( field[f + 2*dir] == color );
-		CHECK( field[f + 3*dir] == opponent );
-		field[f + 2*dir] = opponent;
-		field[f + 3*dir] = free;
-		break;
-	 case Move::left2:
-		dir2 = direction[m.direction-1];
-		CHECK( field[f + dir] == free );
-		CHECK( field[f + dir2] == color );
-		CHECK( field[f + dir+dir2] == color );
-		field[f+dir2] = free;
-		field[f+=dir] = color;
-		field[f+dir2] = free;
-		break;
-	 case Move::right2:
-		dir2 = direction[m.direction+1];
-		CHECK( field[f + dir] == free );
-		CHECK( field[f + dir2] == color );
-		CHECK( field[f + dir+dir2] == color );
-		field[f+dir2] = free;
-		field[f+=dir] = color;
-		field[f+dir2] = free;
-		break;
-	 case Move::move1:       /* (. c) => (c .) */
-		CHECK( field[f + dir] == color );
-		field[f + dir] = free;
-		break;
-	default:
-	  break;
-	}
-
-	if (--storedLast < 0) storedLast = MvsStored-1;
-	
-	CHECK( isConsistent() );
+  int f, dir, dir2;
+  int opponent = color;
+  Move& m = storedMove[storedLast];
+  
+  CHECK( isConsistent() );
+  
+  if (storedFirst == storedLast) return false;
+  
+  /* change actual color */
+  color = (color == color1) ? color2:color1;	
+  
+  if (m.isOutMove()) {
+    if (color == color1)
+      color2Count++;
+    else
+      color1Count++;
+  }       
+  
+  f = m.field;
+  CHECK( field[f] == free );
+  field[f] = color;
+  dir = direction[m.direction];
+  
+  switch(m.type) {
+  case Move::out2:        /* (. c c c o |) => (c c c o o |) */
+    CHECK( field[f + dir] == color );
+    CHECK( field[f + 2*dir] == color );
+    CHECK( field[f + 3*dir] == color );
+    CHECK( field[f + 4*dir] == opponent );
+    CHECK( field[f + 5*dir] == out );
+    field[f + 3*dir] = opponent;
+    break;
+  case Move::out1with3:   /* (. c c c |) => (c c c o |) */
+    CHECK( field[f + dir] == color );
+    CHECK( field[f + 2*dir] == color );
+    CHECK( field[f + 3*dir] == color );
+    CHECK( field[f + 4*dir] == out );
+    field[f + 3*dir] = opponent;
+    break;
+  case Move::move3:       /* (. c c c) => (c c c .)     */
+    CHECK( field[f + dir] == color );
+    CHECK( field[f + 2*dir] == color );
+    CHECK( field[f + 3*dir] == color );
+    field[f + 3*dir] = free;
+    break;
+  case Move::out1with2:   /* (. c c | ) => (c c o |)     */
+    CHECK( field[f + dir] == color );
+    CHECK( field[f + 2*dir] == color );
+    CHECK( field[f + 3*dir] == out );
+    field[f + 2*dir] = opponent;
+    break;
+  case Move::move2:       /* (. c c) => (c c .)       */
+    CHECK( field[f + dir] == color );
+    CHECK( field[f + 2*dir] == color );
+    field[f + 2*dir] = free;
+    break;
+  case Move::push2:       /* (. c c c o o) => (c c c o o .) */
+    CHECK( field[f + dir] == color );
+    CHECK( field[f + 2*dir] == color );
+    CHECK( field[f + 3*dir] == color );
+    CHECK( field[f + 4*dir] == opponent );
+    CHECK( field[f + 5*dir] == opponent );
+    field[f + 3*dir] = opponent;
+    field[f + 5*dir] = free;
+    break;
+  case Move::left3:
+    dir2 = direction[m.direction-1];
+    CHECK( field[f + dir] == free );
+    CHECK( field[f + 2*dir] == free );
+    CHECK( field[f + dir2] == color );
+    CHECK( field[f + dir+dir2] == color );
+    CHECK( field[f + 2*dir+dir2] == color );
+    field[f+dir2] = free;
+    field[f+=dir] = color;
+    field[f+dir2] = free;
+    field[f+=dir] = color;
+    field[f+dir2] = free;
+    break;
+  case Move::right3:
+    dir2 = direction[m.direction+1];
+    CHECK( field[f + dir] == free );
+    CHECK( field[f + 2*dir] == free );
+    CHECK( field[f + dir2] == color );
+    CHECK( field[f + dir+dir2] == color );
+    CHECK( field[f + 2*dir+dir2] == color );
+    field[f+dir2] = free;
+    field[f+=dir] = color;
+    field[f+dir2] = free;
+    field[f+=dir] = color;
+    field[f+dir2] = free;
+    break;
+  case Move::push1with3:   /* (. c c c o) => (c c c o .) */
+    CHECK( field[f + dir] == color );
+    CHECK( field[f + 2*dir] == color );
+    CHECK( field[f + 3*dir] == color );
+    CHECK( field[f + 4*dir] == opponent );
+    field[f + 3*dir] = opponent;
+    field[f + 4*dir] = free;
+    break;
+  case Move::push1with2:   /* (. c c o) => (c c o .) */
+    CHECK( field[f + dir] == color );
+    CHECK( field[f + 2*dir] == color );
+    CHECK( field[f + 3*dir] == opponent );
+    field[f + 2*dir] = opponent;
+    field[f + 3*dir] = free;
+    break;
+  case Move::left2:
+    dir2 = direction[m.direction-1];
+    CHECK( field[f + dir] == free );
+    CHECK( field[f + dir2] == color );
+    CHECK( field[f + dir+dir2] == color );
+    field[f+dir2] = free;
+    field[f+=dir] = color;
+    field[f+dir2] = free;
+    break;
+  case Move::right2:
+    dir2 = direction[m.direction+1];
+    CHECK( field[f + dir] == free );
+    CHECK( field[f + dir2] == color );
+    CHECK( field[f + dir+dir2] == color );
+    field[f+dir2] = free;
+    field[f+=dir] = color;
+    field[f+dir2] = free;
+    break;
+  case Move::move1:       /* (. c) => (c .) */
+    CHECK( field[f + dir] == color );
+    field[f + dir] = free;
+    break;
+  default:
+    break;
+  }
+  
+  if (--storedLast < 0) storedLast = MvsStored-1;
+  
+  CHECK( isConsistent() );
+  
+  return true;
 }
 
 int Board::movesStored()
@@ -636,272 +646,214 @@ int Board::movesStored()
   return c;
 }
 
-void Board::showMove(const Move& mm, int step)
-{
-	int f, dir, dir2;
-	int opponentNew, colorNew;
-	bool afterMove;
-	static Move lastMove;
-	Move m;
 
-	if (boardOk) {
-	  /* board ok means: board has the normal state before move */
-	  CHECK( isConsistent() );
-	  if (step == 0)
-	    return;      /* nothing to be done */
-	}
-	boardOk = (step == 0) ? true:false;
-
-	if (step == 0) 
-	  m = lastMove;
-	else {
-	  m = lastMove = mm;
-	}
-
-	if (color == color1) {
-	  colorNew = (step<2) ? color1 : ((step>2) ? color1bright:free );
-	  opponentNew = (step<2) ? color2 : color2bright;
-	}
-	else {
-	  colorNew = (step<2) ? color2 : ((step>2) ? color2bright:free );
-	  opponentNew = (step<2) ? color1 : color1bright;
-	}
-
-	afterMove = (step == 1) || (step == 4);
-
-	f = m.field;
-	CHECK( (m.type >= 0) && (m.type < Move::none));
-	dir = direction[m.direction];
-
-	/* first field */
-	field[f] = afterMove ? free : colorNew;
-
-	switch(m.type) {
-	case Move::out2:        /* (c c c o o |) */
-	  field[f + dir] = colorNew;
-	  field[f + 2*dir] = colorNew;
-	  field[f + 3*dir] = afterMove ? colorNew : opponentNew;
-	  field[f + 4*dir] = opponentNew;
-	  break;		
-	case Move::out1with3:   /* (c c c o |)   */
-	  field[f + dir] = colorNew;
-	  field[f + 2*dir] = colorNew;
-	  field[f + 3*dir] = afterMove ? colorNew : opponentNew;
-	  break;
-	case Move::move3:       /* (c c c .)     */
-	  field[f + dir] = colorNew;
-	  field[f + 2*dir] = colorNew;
-	  field[f + 3*dir] = afterMove ? colorNew : free;
-	  break;
-	case Move::out1with2:   /* (c c o |)     */
-	  field[f + dir] = colorNew;
-	  field[f + 2*dir] = afterMove ? colorNew : opponentNew;
-	  break;
-	case Move::move2:       /* (c c .)       */
-	  field[f + dir] = colorNew;
-	  field[f + 2*dir] = afterMove ? colorNew : free;
-	  break;
-	case Move::push2:       /* (c c c o o .) */
-	  field[f + dir] = colorNew;
-	  field[f + 2*dir] = colorNew;
-	  field[f + 3*dir] = afterMove ? colorNew : opponentNew;
-	  field[f + 4*dir] = opponentNew;
-	  field[f + 5*dir] = afterMove ? opponentNew : free;
-	  break;
-	case Move::left3:		
-	  dir2 = direction[m.direction-1];
-	  field[f+dir2] = afterMove ? colorNew : free;
-	  field[f+=dir] = afterMove ? free : colorNew;
-	  field[f+dir2] = afterMove ? colorNew : free;
-	  field[f+=dir] = afterMove ? free : colorNew;
-	  field[f+dir2] = afterMove ? colorNew : free;
-	  break;
-	case Move::right3:
-	  dir2 = direction[m.direction+1];
-	  field[f+dir2] = afterMove ? colorNew : free;
-	  field[f+=dir] = afterMove ? free : colorNew;
-	  field[f+dir2] = afterMove ? colorNew : free;
-	  field[f+=dir] = afterMove ? free : colorNew;
-	  field[f+dir2] = afterMove ? colorNew : free;
-	  break;
-	case Move::push1with3:   /* (c c c o .) => (. c c c o) */
-	  field[f + dir] = colorNew;
-	  field[f + 2*dir] = colorNew;
-	  field[f + 3*dir] = afterMove ? colorNew : opponentNew;
-	  field[f + 4*dir] = afterMove ? opponentNew : free;
-	  break;
-	case Move::push1with2:   /* (c c o .) => (. c c o) */
-	  field[f + dir] = colorNew;
-	  field[f + 2*dir] = afterMove ? colorNew : opponentNew;
-	  field[f + 3*dir] = afterMove ? opponentNew : free;
-	  break;
-	case Move::left2:
-	  dir2 = direction[m.direction-1];
-	  field[f+dir2] = afterMove ? colorNew : free;
-	  field[f+=dir] = afterMove ? free : colorNew;
-	  field[f+dir2] = afterMove ? colorNew : free;
-	  break;
-	case Move::right2:
-	  dir2 = direction[m.direction+1];
-	  field[f+dir2] = afterMove ? colorNew : free;
-	  field[f+=dir] = afterMove ? free : colorNew;
-	  field[f+dir2] = afterMove ? colorNew : free;
-	  break;
-	case Move::move1:       /* (c .) => (. c) */
-	  field[f + dir] = afterMove ? colorNew : free;
-	  break;
-	default:
-	  break;
-	}
-}
-
-void Board::showStart(const Move& m, int step)
-{
-	int f, dir;
-	int colorNew;
-
-	if (boardOk) {
-	  /* board ok means: board has the normal state before move */
-	  CHECK( isConsistent() );
-	  if (step == 0)
-	    return;      /* nothing to be done */
-	}
-	boardOk = (step == 0) ? true:false;
-
-	if (color == color1)
-	  colorNew = (step==0) ? color1 : color1bright;
-	else 
-	  colorNew = (step==0) ? color2 : color2bright;
-
-	f = m.field;
-
-	/* first field */
-	field[f] = colorNew;
-
-	switch(m.type) {
-	case Move::left3:
-	case Move::right3:
-	  dir = direction[m.direction];
-	  field[f + dir] = colorNew;
-	  field[f + 2*dir] = colorNew;
-	  break;
-	case Move::left2:
-	case Move::right2:
-	  dir = direction[m.direction];
-	  field[f + dir] = colorNew;
-	default:
-	  break;
-	}
-}
-
-
-/* calc value out of possible moves starting at field <startField>
- * parameter color shadows class member variable !
+/** countFrom
+ *
+ * Used for board evaluation to count allowed move types and
+ * connectiveness. VERY similar to move generation.
+ *
+ * Returns number of found moves
  */
-int Board::calcFieldValue(int startField, int color, int value)
+void Board::countFrom(int startField, int color, 
+		      MoveTypeCounter& TCounter,
+		      ConnectCounter& CCounter)
 {
-	int d, dir, c, actField;
-	int opponent = (color == color1) ? color2 : color1;
+  int d, dir, c, actField, c2;
+  bool left, right;
+  
+  /* 6 directions	*/
+  for(d=1;d<7;d++) {
+    dir = direction[d];
+    
+    /* 2nd field */
+    c = field[actField = startField+dir];
+    if (c == free) {
+      TCounter.incr( Move::move1 );
+      continue;
+    }
 
-	/* 6 directions	*/
-	for(d=1;d<7;d++) {
-		dir = direction[d];
-		  
-		/* 2nd field */
-		if (field[actField = startField+dir] != color)
-		  continue;
-		
-		/* 2nd == color */
-		
-		/* 3rd field */
-		c = field[actField += dir];
-		if (c == free) {
-			/* (c c .) */
-			value+=moveValue[0];
-			continue;
-		}		
-		else if (c == opponent) {
-			
-			/* 4th field */
-			c = field[actField += dir];
-			if (c == free) {
-			  /* (c c o .) */
-			  value+=moveValue[2];
-			}
-			else if (c == out) {
-			  /* (c c o |) */
-			  value+=moveValue[4];
-			}
-			continue;
-		}
-		if (c != color)
-		  continue;
-		
-		/* 3nd == color */
+    if (c != color)
+      continue;
+    
+    /* 2nd == color */
+    
+    CCounter.incr( ConnectCounter::connect2 );
 
-		/* 4th field */
-		c = field[actField += dir];
-		if (c == free) {
-		  /* (c c c .) */
-		  value+=moveValue[1];
-		  continue;
-		}
-		if (c != opponent)
-		  continue;
+    /* left side move 2 */
+    left = (field[startField+direction[d-1]] == free);
+    if (left) {
+      left = (field[actField+direction[d-1]] == free);
+      if (left) 
+	TCounter.incr( Move::left2 );
+    }
 
-		/* 4nd == opponent */
-		
-		/* 5. field */
-		c = field[actField += dir];
-		if (c == free) {
-		  /* (c c c o .) */
-		  value+=moveValue[2];
-		  continue;
-		}
-		else if (c == out) {
-		  /* (c c c o |) */
-		  value+=moveValue[4];
-		  continue;
-		}
-		if (c != opponent)
-		  continue;
+    /* right side move 2 */
+    right = (field[startField+direction[d+1]] == free);
+    if (right) {
+      right = (field[actField+direction[d+1]] == free);
+      if (right) 
+	TCounter.incr( Move::right2 );
+    }
 
-		/* 5nd == opponent */
+    /* 3rd field */
+    c = field[actField += dir];
+    if (c == free) {
+      /* (c c .) */
+      TCounter.incr( Move::move2 );
+      continue;
+    }
+    else if (c == out) {
+      continue;
+    }
+    else if (c != color) {
+      
+      /* 4th field */
+      c = field[actField += dir];
+      if (c == free) {
+	/* (c c o .) */
+	TCounter.incr( Move::push1with2 );
+      }
+      else if (c == out) {
+	/* (c c o |) */
+	TCounter.incr( Move::out1with2 );
+      }
+      continue;
+    }
+    
+    /* 3nd == color */
 
-		/* 6. field */
-		c = field[actField += dir];
-		if (c == free) {
-		  /* (c c c o o .) */
-		  value+=moveValue[3];
-		}
-		else if (c == out) {
-		  /* (c c c o o |) */
-		  value+=moveValue[4];
-		}
-	}
-	return value;
+    CCounter.incr( ConnectCounter::connect3 );
+
+    /* left side move 3 */
+    if (left) {
+      left = (field[actField+direction[d-1]] == free);
+      if (left) 
+	TCounter.incr( Move::left3 );
+    }
+
+    /* right side move 3 */
+    if (right) {
+      right = (field[actField+direction[d+1]] == free);
+      if (right) 
+	TCounter.incr( Move::right3 );
+    }
+
+    /* 4th field */
+    c = field[actField += dir];
+    if (c == free) {
+      /* (c c c .) */
+      TCounter.incr( Move::move3 );
+      continue;
+    }
+    else if (c == out) {
+      continue;
+    }
+    else if (c != color) {
+
+      /* 4nd == opponent */
+
+      /* 5. field */
+      c2 = field[actField += dir];
+      if (c2 == free) {
+	/* (c c c o .) */
+	TCounter.incr( Move::push1with3 );
+	continue;
+      }
+      else if (c2 == out) {
+	/* (c c c o |) */
+	TCounter.incr( Move::out1with3 );
+	continue;
+      }
+      if (c2 != c)
+	continue;
+
+      /* 5nd == opponent */
+
+      /* 6. field */
+      c2 = field[actField += dir];
+      if (c2 == free) {
+	/* (c c c o o .) */
+	TCounter.incr( Move::push2 );
+      }
+      else if (c2 == out) {
+	/* (c c c o o |) */
+	TCounter.incr( Move::out2 );
+      }
+
+      continue;
+    }
+
+    /* 4nd == color */
+
+    CCounter.incr( ConnectCounter::connect4 );
+
+    /* 5th field */
+    c = field[actField += dir];
+    if (c != color)
+      continue;
+
+    /* 4nd == color */
+
+    CCounter.incr( ConnectCounter::connect5 );
+  }
+}
+
+/** indent
+ *
+ * Internal: for debugging output only
+ */
+void indent(int d)
+{
+	char tmp[]="                                        ";
+	tmp[d*3] = 0;
+	printf(">        %s",tmp);
 }
 
 
+
+/** validState
+ *
+ * Check for a valid board position to play from:
+ * (1) Number of balls for each color has to be between 9 and 14
+ * (2) There must be a move possible for actual color
+ */
 int Board::validState()
 {
+  MoveTypeCounter tc;
+  ConnectCounter  cc;
+
   int c1 = 0, c2 = 0;
-  int i,j;
+  int i,j, moveCount, res;
        
   for(i=0;i<AllFields;i++) {              
     j=field[i];
     if (j == color1) c1++;
     if (j == color2) c2++;
+    if (j == color)
+      countFrom( i, color, tc, cc);
   }
 
   color1Count = c1;
   color2Count = c2;
+  moveCount = tc.sum();
 
   if (c1==0 && c2==0) 
-    return empty;
-  if (c1>8 && c1<15 && c2>8 && c2<15) 
-    return valid;
-  return invalid;
+    res = empty;
+  else if (c1>8 && c1<15 && c2>8 && c2<15 && moveCount>0 ) 
+    res = valid;
+  else 
+    res = invalid;
+
+#ifdef MYTRACE
+  if (spyLevel>2) {
+    indent(spyDepth);
+    printf("Valid: %s (Color1 %d, Color2 %d, moveCount of %d: %d)\n",
+	   (res == empty) ? "empty" : (res==valid) ? "valid":"invalid",
+	   c1,c2,color,moveCount);
+  }
+#endif
+
+  return res;
 }
 
 
@@ -914,25 +866,72 @@ int Board::validState()
  */
 int Board::calcValue()
 {
-        int value=0,f;
-	int i,j;
-	
-	for(i=0;i<RealFields;i++) {		  
-		j=field[f=order[i]];
-		if (j == 0) continue;
-		if (j == color) 
-		  value -= calcFieldValue(f,j,fieldValue[i]);
-		else
-		  value += calcFieldValue(f,j,fieldValue[i]);
-	}
-	if (color == color2)
-	  value += stoneValue[14 - color1Count] - 
-                   stoneValue[14 - color2Count];
-	else
-	  value += stoneValue[14 - color2Count] - 
-                   stoneValue[14 - color1Count];
+  MoveTypeCounter tcColor, tcOpponent;
+  ConnectCounter  ccColor, ccOpponent;
 
-	return value;
+  int f,i,j;
+
+  /* different evaluation types */
+  int fieldValueSum=0, stoneValueSum=0;
+  int moveValueSum=0, connectValueSum=0;
+  int valueSum;
+
+  /* First check simple winner condition */
+  if (color1Count <9)
+    valueSum = (color==color1) ? 16000 : -16000;
+  else if (color2Count <9)
+    valueSum = (color==color2) ? 16000 : -16000;
+  else {
+
+    /* Calculate fieldValueSum and count move types and connectivity */
+    for(i=0;i<RealFields;i++) {		  
+      j=field[f=order[i]];
+      if (j == free) continue;
+      if (j == color) {
+	countFrom( f, j, tcColor, ccColor );
+	fieldValueSum -= fieldValue[i];
+      }
+      else {
+	countFrom( f, j, tcOpponent, ccOpponent );
+	fieldValueSum += fieldValue[i];
+      }
+    }
+    
+    /* If color can't do any moves, opponent wins... */
+    if (tcColor.sum() == 0) 
+      valueSum = 16000;
+    else {
+
+      for(int i=0;i < Move::typeCount;i++)
+	moveValueSum += moveValue[i] * 
+	  (tcOpponent.get(i) - tcColor.get(i)) /2;
+      
+      for(int i=0;i < ConnectCounter::connectCount;i++)
+	connectValueSum += connectValue[i] * 
+	  (ccOpponent.get(i) - ccColor.get(i)) /2;
+      
+      if (color == color2)
+	stoneValueSum = stoneValue[14 - color1Count] - 
+	  stoneValue[14 - color2Count];
+      else
+	stoneValueSum = stoneValue[14 - color2Count] - 
+	  stoneValue[14 - color1Count];
+      
+      valueSum = fieldValueSum + moveValueSum + 
+	connectValueSum + stoneValueSum;
+    }
+  }
+
+#ifdef MYTRACE
+  if (spyLevel>2) {
+    indent(spyDepth);
+    printf("Eval %d (field %d, move %d, connect %d, stone %d)\n",
+	   valueSum, fieldValueSum, moveValueSum,
+	   connectValueSum, stoneValueSum );
+  }
+#endif
+
+  return valueSum;
 }
 
 bool Board::isConsistent()
@@ -996,19 +995,13 @@ void Board::showHist()
 }
 */
 
-void indent(int d)
-{
-	char tmp[]="                                        ";
-	tmp[d*3] = 0;
-	printf(">        %s",tmp);
-}
 
 /*
  * We always try the maximize the board value
  */
 int Board::search(int depth, int alpha, int beta)
 {
-	int actValue=-16000, value;
+	int actValue= -14999+depth, value;
 	Move m;
 	MoveList list;
 	bool stop = false;
@@ -1024,6 +1017,8 @@ int Board::search(int depth, int alpha, int beta)
 
 #ifdef MYTRACE
 
+	spyDepth = depth;
+
 	moveCount += list.getLength();
 
 	/*
@@ -1038,21 +1033,21 @@ int Board::search(int depth, int alpha, int beta)
 #endif
 	
 	/* check for a old best move in main combination */
-	if (inMainCombination) {
-	  m = mc[depth];
+	if (inPrincipalVariation) {
+	  m = pv[depth];
 	  if (!list.isElement(m, 0, true)) 
 	    m.type = Move::none;
 	  if (m.type == Move::none)
-	    inMainCombination = false;
+	    inPrincipalVariation = false;
 #ifdef MYTRACE
 	  else {
 	    if (spyLevel>1) {		 
-	      indent(depth);
-	      printf("Got from MC !\n" );
+	      indent(spyDepth);
+	      printf("Got from pv !\n" );
 	    }
 	  }
 #endif
-	  /* for moves in MainCombination we do depth search
+	  /* for moves in PrincipalVariation we do depth search
 	   * even if we don't normally in this depth => Comment out
 	   */
 	  //if (m.type > maxType)
@@ -1080,7 +1075,7 @@ int Board::search(int depth, int alpha, int beta)
 	  int oldCutoffCount = cutoffCount;
 	  
 	  if (spyLevel>1) {
-	    indent(depth);
+	    indent(spyDepth);
 	    printf("%s [%6d .. %6d] ",
 		   (color==color1)?"O":"X", alpha,beta);
 	    m.print();
@@ -1093,6 +1088,7 @@ int Board::search(int depth, int alpha, int beta)
 #endif
 	  playMove(m);
 	  if (!isValid()) {
+	    /* Possibility (1) to win: Ball Count <9 */
 	    value = 14999-depth;
 	    //  value = ((depth < maxDepth) ? 15999:14999) - depth;
 #ifdef MYTRACE
@@ -1101,7 +1097,7 @@ int Board::search(int depth, int alpha, int beta)
 	  }
 	  else {
 	    
-	    /* opponent searches for his maximum; but we won't the
+	    /* opponent searches for his maximum; but we want the
 	     * minimum: so change sign (for alpha/beta window too!)
 	     */		  
 	    value = - search(depth+1,-beta,-alpha);
@@ -1115,7 +1111,7 @@ int Board::search(int depth, int alpha, int beta)
 #ifdef MYTRACE	
 	  if (spyLevel>1) {
 
-	    indent(depth);
+	    indent(spyDepth);
 	    if (oldSearchCalled < searchCalled) {
 	      printf("  %d Calls", searchCalled-oldSearchCalled);
 	      if (cutoffCount>oldCutoffCount)
@@ -1131,7 +1127,7 @@ int Board::search(int depth, int alpha, int beta)
 	      if (wonPositions > oldWonPositions)
 		printf(" (%d Won)", wonPositions- oldWonPositions);
 	      printf("\n");
-	      indent(depth);
+	      indent(spyDepth);
 	    }
 	    
 	    printf("  => Rated %d%s\n",
@@ -1153,17 +1149,17 @@ int Board::search(int depth, int alpha, int beta)
 #endif
 	  if (value > actValue) {
 	    actValue = value;
-	    mc.update(depth, m);
+	    pv.update(depth, m);
 	    
 	    if (bUpdateSpy && depth == 0) {
 	      emit updateBestMove(m, actValue);
 #ifdef MYTRACE	
 	      if (spyLevel>0) {
 		int i;
-		printf(">      New MC (Rating %d):", actValue);
+		printf(">      New pv (Rating %d):", actValue);
 		for(i=0;i<=maxDepth;i++) {
 		  printf("\n>          D %d: %s", 
-			 i, (const char*)(mc[i].name()) );
+			 i, (const char*)(pv[i].name()) );
 		}
 		printf("\n>\n");
 	      }
@@ -1212,7 +1208,7 @@ int Board::search(int depth, int alpha, int beta)
 
 #ifdef MYTRACE
 	  if (spyLevel>2) {
-	    indent(depth);
+	    indent(spyDepth);
 	    printf("%s (%6d .. %6d) %-25s => Rating %6d%s\n",
 		   (color==color1)?"O":"X", alpha,beta,
 		   (const char*)m.name(),
@@ -1226,7 +1222,22 @@ int Board::search(int depth, int alpha, int beta)
 #endif
 	  if (value > actValue) {
 	    actValue = value;
-	    mc.update(depth, m);
+	    pv.update(depth, m);
+
+	    if (bUpdateSpy && depth == 0) {
+	      emit updateBestMove(m, actValue);
+#ifdef MYTRACE	
+	      if (spyLevel>0) {
+		int i;
+		printf(">      New pv (Rating %d):", actValue);
+		for(i=0;i<=maxDepth;i++) {
+		  printf("\n>          D %d: %s", 
+			 i, (const char*)(pv[i].name()) );
+		}
+		printf("\n>\n");
+	      }
+#endif
+	    }
 	    
 	    if (actValue>14900 || actValue >= beta)
 	      return actValue;
@@ -1243,11 +1254,12 @@ Move& Board::bestMove()
 	int alpha=-15000,beta=15000;
 	int nalpha,nbeta, actValue;
 	
-	mc.clear(realMaxDepth);
+	pv.clear(realMaxDepth);
 	
 	maxDepth=1;
 	show = false;
 	breakOut = false;
+	spyDepth = 0;
 
 	if (spyLevel>0)
 	  printf("\n> New Search\n>\n");
@@ -1262,7 +1274,7 @@ Move& Board::bestMove()
 		    printf(">     AB-Window: (%d ... %d)\n>\n", alpha, beta);
 
 		  nalpha=alpha, nbeta=beta;
-		  inMainCombination = (mc[0].type != Move::none);
+		  inPrincipalVariation = (pv[0].type != Move::none);
 
 		  /* Statistics */
 		  searchCalled = 0;
@@ -1281,10 +1293,10 @@ Move& Board::bestMove()
 		      int i;
 		      if (spyLevel>1)
 			printf(">\n");
-		      printf(">      Got MC with Rating %d:",actValue);
+		      printf(">      Got PV with Rating %d:",actValue);
 		      for(i=0;i<=maxDepth;i++) {
 			printf("\n>          D %d: ", i);
-			mc[i].print();
+			pv[i].print();
 		      }
 		      printf("\n>\n");
 
@@ -1300,8 +1312,9 @@ Move& Board::bestMove()
 		    }
 
 		  /* Don't break out if we haven't found a move */
-		  if (mc[0].type == Move::none) 
+		  if (pv[0].type == Move::none) 
 		    breakOut=false;
+		    
 		  if (actValue > 14900 || actValue < -14900)
 		    breakOut=true;
 		  if (actValue <= nalpha)
@@ -1323,7 +1336,7 @@ Move& Board::bestMove()
 		/*
 		  if (spyLevel>0) {
 		  printf(">>> Best Move so far: %d - ", actValue);
-		  mc[0].print();
+		  pv[0].print();
 		  printf("\n");
 		  }	
 		*/	
@@ -1335,11 +1348,13 @@ Move& Board::bestMove()
 	  changeRating();
 	else {
 	  printf(">>> Got Move : ");
-	  mc[0].print();
+	  pv[0].print();
 	  printf("\n\n");
 	}
 
-	return(mc[0]);
+	spyDepth = 0;
+
+	return(pv[0]);
 }	  
 
 Move Board::randomMove()
@@ -1428,7 +1443,7 @@ int Board::setASCIIState(const QString& state)
     moveNo = state.mid(index+1,3).toInt();
 
   int f=12, row=0, rowEnd = 17;
-  char c;
+  char c = ' ';
 
   index=state.find("/");
 
@@ -1485,12 +1500,8 @@ QString Board::getState(int moveNo)
 {
   QString state;
   QString entry, tmp;
-  Move m;
   int i;
   
-  /* remove any highlighting */
-  showMove(m,0);
-  
   /* Color + Counts */
   state += (char) ('A' + moveNo /25 );
   state += (char) ('A' + moveNo %25 );
