@@ -4,7 +4,7 @@
 
 #include <kstatusbar.h>
 #include <ktoolbar.h>
-#include <ktmainwindow.h>
+#include <kmainwindow.h>
 #include <kmenubar.h>
 #include <kaccel.h>
 #include <qdict.h>
@@ -13,10 +13,12 @@
 #include "view.h"
 #include "highscore.h"
 
+//#define KA_ENABLE_SOUND
+
 class KALedMeter;
 class QLCDNumber;
 
-class KAstTopLevel : public KTMainWindow
+class KAstTopLevel : public KMainWindow
 {
     Q_OBJECT
 public:
@@ -49,7 +51,6 @@ private slots:
     void slotKeyConfig();
     void slotPref();
     void slotShowHighscores();
-    void slotReturn();  // returning from dialogs - unpausing the game really...
 
 private:
     KAsteroidsView *view;
