@@ -20,19 +20,21 @@ class Status : public QWidget
  public :
 	Status(QWidget *parent=0, const char *name=0);
 
-	bool newGame(GameType &);
+	void newGame(const Level &);
+	const Level &currentLevel() const { return field->level(); }
 
  signals:
 	void keyboardEnabled(bool);
+	void message(const QString &);
 	void gameStateChanged(GameState);
 	
  public slots:
 	void restartGame();
-	void changeCase(uint i, uint j);
+	void changeCase(CaseState, uint inc);
 	void update(bool);
-	void endGame(int);
-	void showHighScores() { highScores(0); }
-	void pauseGame() { field->pause(); }
+	void endGame()        { _endGame(false); }
+	void showHighScores();
+	void pauseGame()      { field->pause(); }
 	void print();
 	void preferences();
 	void setGameState(GameState);
@@ -51,12 +53,12 @@ class Status : public QWidget
 	
 	Smiley       *smiley;
 	LCDNumber    *left;
-	QLabel       *message;
 	DigitalClock *dg;
 
 	void exmesg(const QString &);
 	void highScores(const Score *);
 	void initGame();
+	void _endGame(bool win);
 };
 
 #endif // STATUS_H
