@@ -17,21 +17,18 @@
 #ifndef __PokerWindow__
 #define __PokerWindow__
 
-#include <ktmainwindow.h>
-#include <qwidget.h>
-
+#include <kmainwindow.h>
 
 class KMenuBar;
 class KToolBar;
 class KStatusBar;
 class kpok;
 
-
-class PokerWindow : public KTMainWindow
+class PokerWindow : public KMainWindow
 {
 	Q_OBJECT
 private:
-	KMenuBar        *menu;
+//	KMenuBar        *menu;
 
 	QPopupMenu	*filePopup;
 	QPopupMenu	*optionsPopup;
@@ -47,6 +44,7 @@ protected slots:
 	void commandCallback(int id);
 	void setHand(QString newHand, bool lastHand = true);
 	void showClickToHold(bool show);
+	void saveOptions();
 	
 public:
 	PokerWindow();
