@@ -36,20 +36,33 @@
  *******************************************************************
  */
 
+#include <kapp.h>
+#include <kstddirs.h>
+#include <kglobal.h>
+#include <kimageio.h>
+#include <klocale.h>
+#include <kcmdlineargs.h>
+#include <kaboutdata.h>
 
 #include "misc.h"
-#include <kapp.h>
+#include "version.h"
 #include "app.h"
 
-QString PICDIR;
 
-int main(int argc, char **argv) {
-  KApplication a(argc, argv, "kreversi");
+static const char *description = I18N_NOOP("KDE Board Game");
 
-  // locate the picture directory
-  PICDIR.setStr(KApplication::kde_datadir().copy());
-  PICDIR.append("/kreversi/pics/");
+int main(int argc, char **argv)
+{
+  KAboutData aboutData( "kreversi", I18N_NOOP("KReversi"), 
+    KREVERSI_VERSION, description, KAboutData::License_GPL, 
+    "(c) 1997-2000, Mario Weilguni");
+  aboutData.addAuthor("Mario Weilguni",0, "mweilguni@sime.com");
+  KCmdLineArgs::init( argc, argv, &aboutData );
 
+  KApplication a;
+
+  KImageIO::registerFormats();
+  
   App *app = new App;
 
   app->show();
