@@ -24,8 +24,8 @@
 
 QBitArray gameState(5);
 QLabel *label = 0;
-int speed[4] = { 85, 75, 55, 40 };
-
+//int speed[4] = { 85, 75, 55, 40 };
+int speed[4] = { 110, 85, 55, 40 };
 Rattler::Rattler( QWidget *parent, const char *name )
     : QWidget( parent, name )
 {
@@ -45,7 +45,6 @@ Rattler::Rattler( QWidget *parent, const char *name )
     level = new Level(board);
 
     pix = new PixServer(board, this);
-    // setFixedSize(pix->levelPix().size());
 
     basket = new Basket(board, pix);
     samy = new SamySnake(board, pix);
@@ -65,25 +64,21 @@ Rattler::Rattler( QWidget *parent, const char *name )
     gameState.setBit(Demo);
 
     timerCount = 0;
-    QTimer::singleShot( 2000, this, SLOT(demo()) );
+    QTimer::singleShot( 2000, this, SLOT(demo()) ); // Why wait?
 
 }
 
 void Rattler::paintEvent( QPaintEvent *e)
 {
     QRect rect = e->rect();
-    bool dirty = FALSE;
 
     if (rect.isEmpty())
       return;
-    dirty = TRUE;
     QPixmap levelPix = pix->levelPix();
+
     bitBlt(this, rect.x(), rect.y(),
               &levelPix, rect.x(), rect.y(), rect.width(), rect.height());
     //most Below here was moved to timerEvent where it belongs...
-
-
-
 }
 
 void Rattler::timerEvent( QTimerEvent * )
@@ -93,33 +88,28 @@ void Rattler::timerEvent( QTimerEvent * )
     if ( !leaving )		// advance progressBar unless Samy
 	emit advance();		// is going out
 
-    for (CompuSnake *c = computerSnakes->first(); c != 0;
-	 c = computerSnakes->next())
-	if(c) c->nextMove();
+    for (CompuSnake *c = computerSnakes->first(); c != 0; c = computerSnakes->next())
+	if(c) {
+                c->nextMove();
+                c->repaint(false);
+                }
+
+    for (Ball *b = balls->first(); b != 0; b = balls->next())
+	if (b) {
+                b->nextMove();
+                b->repaint();
+                }
 
-    for (Ball *b = balls->first(); b != 0;
-	 b = balls->next())
-	if (b) b->nextMove();
 
     samyState state = ok;
 
     if(!gameState.testBit(Demo))
+    {
 	state = samy->nextMove(direction);
+        samy->repaint( false );
+    }
 
-    //    repaint(0,0,1,1, FALSE);
-    // Below here was moved from paintEvent, which was causing unnecessary paints....
-
-	if(!gameState.testBit(Demo))
-	  samy->repaint( false );
-        basket->repaint( false);
-	for (CompuSnake *as = computerSnakes->first(); as != 0;
-	     as = computerSnakes->next())
-	    if (as) as->repaint(false);
-
-	for (Ball *b = balls->first(); b != 0;
-	     b = balls->next())
-	    if (b) b->repaint();
-
+    basket->repaint( false); //whats 'basket'?
 
     if (state == ko)
 	newTry();
@@ -170,31 +160,6 @@ void Rattler::keyPressEvent( QKeyEvent *k )
     }
     k->accept();
 
-    /*
-    switch ( k->key()) {
-    case (const int)UpKey:
-	direction = N;
-	break;
-    case  DownKey:
-	direction = S;
-	break;
-    case RightKey:
-	direction = E;
-	break;
-    case LeftKey:
-	direction = W;
-	break;
-    default:
-	k->ignore();
-	return;
-    }
-    k->accept();
-    */
-
-
-
-
-
 }
 
 void Rattler::closeGate(int i)
@@ -277,7 +242,7 @@ void Rattler::pause()
 	stop();
 
 	label = new QLabel(this);
-	label->setFont( QFont( "Times", 16, QFont::Bold ) );
+	label->setFont( QFont( "Times", 14, QFont::Bold ) );
 	label->setText(i18n("Game Paused\n Press F3 to resume\n"));
 	label->setAlignment( AlignCenter );
 	label->setFrameStyle( QFrame::Panel | QFrame::Raised );
@@ -353,11 +318,11 @@ void Rattler::restart()
 	switch( KMessageBox::questionYesNo( this,
 					    i18n("A game is already started.\n"
 						 "Start a new one?\n"),  i18n("Snake Race"))) {
-	case KMessageBox::No:
+	case KMessageBox::Yes:
 	    if ( gameState.testBit(Paused))
 		emit togglePaused();
 	    break;
-	case KMessageBox::Yes:
+	case KMessageBox::No:
 	    if ( !gameState.testBit(Paused))
 		start();
 	    return;
