@@ -8,18 +8,18 @@
 #include <qgroupbox.h>
 #include <qlayout.h>
 #include <qvalidator.h>
-#include <qmessagebox.h>
+#include <qvbox.h>
 
+#include <kmessagebox.h>
 #include <kconfig.h>
 #include <klocale.h>
 #include <kapp.h>
 #include <kaccel.h>
-#include <qvbox.h>
 
 #include "dialogs.h"
 
 KeySetup::KeySetup(SOptions *opt,QWidget *parent,const char *name)
-      :KDialogBase( parent, name, true, i18n( "KeySetup" ), Ok|Cancel|Default )
+  : KDialogBase( parent, name, true, i18n( "Key Setup" ), Ok|Cancel|Default|Help )
 {
    int i,p;
    QWidget* topWidget = new QWidget( this );
@@ -28,11 +28,9 @@ KeySetup::KeySetup(SOptions *opt,QWidget *parent,const char *name)
    waitForKey=-1;
    options=opt;
 
-//   setCaption(i18n("Key Setup"));
-
-   for(p=0;p<2;p++)
-      for(i=0;i<PlayerKeyNum;i++)
-         key[p][i]=opt->playerKey[p][i];
+   for( p = 0; p < 2; p++ )
+     for( i = 0; i < PlayerKeyNum; i++ )
+       key[p][i] = opt->playerKey[p][i];
 
    QVBoxLayout *topLayout = new QVBoxLayout( topWidget, 0, spacingHint( ) );
 
@@ -40,7 +38,7 @@ KeySetup::KeySetup(SOptions *opt,QWidget *parent,const char *name)
    bplayer[0]=new QGroupBox(i18n("Red Player"), topWidget );
    bplayer[1]=new QGroupBox(i18n("Blue Player"), topWidget );
  
-   for(p=0;p<2;p++)
+   for( p = 0; p < 2; p++ )
    {
       button[p][PlayerKeyLeft]=new QPushButton(i18n("Rotate Left"),bplayer[p]);
       button[p][PlayerKeyRight]=new QPushButton(i18n("Rotate Right"),bplayer[p]);
@@ -89,6 +87,7 @@ KeySetup::KeySetup(SOptions *opt,QWidget *parent,const char *name)
    players->addWidget(bplayer[1]);
 
    topLayout->activate();
+   setHelp( "Keys" );
 }
 
 void KeySetup::setButtons(int pl,int b)
@@ -138,9 +137,11 @@ void KeySetup::slotOk()
          sameKey=sameKey||(key[1][i]==key[1][j]);
    }
    if(sameKey)
-      sameKey=QMessageBox::warning(this,i18n("Key Setup"),
-	     i18n("There are multiple functions for a key\nContinue?"),
-				   i18n("Yes"), i18n("No"))!=0;
+     sameKey = KMessageBox::warningContinueCancel(this,
+         i18n("There are multiple functions for a key\n"
+              "Continue?"),
+	      i18n("Key Setup"), i18n("&OK"))
+       == KMessageBox::Cancel;
    if(!sameKey)
    {
       for(p=0;p<2;p++)
@@ -224,7 +225,7 @@ void KeySetup::keyPressEvent(QKeyEvent *ev)
    i18n("Reload time"),i18n("Acceleration"),i18n("Energy need"),
    i18n("Rotation speed"),i18n("Energy need"),i18n("Crash damage"),
    i18n("Sun energy"),i18n("Gravity"),i18n("Position X"),i18n("Position Y"),
-   i18n("Velocity X",i18n("Velocity Y"),
+   i18n("Velocity X"),i18n("Velocity Y"),
    i18n("Appearance time"),i18n("Life time"),i18n("Energy Amount"),i18n("Shield Amount")
 #endif
 
@@ -310,7 +311,7 @@ ConfigSetup::ConfigSetup(SConfig *custom,SOptions *opt,
    
    //resize(450,400);
    //setCaption(i18n("Game Setup"));
-   setHelp( "kspaceduel/options.html", "OptionsConfiguration" );
+   setHelp( "OptionsConfigurations" );
    
    customConfig=custom;
    config=*customConfig;
@@ -569,7 +570,7 @@ AiSetup::AiSetup(SOptions *opt,QWidget *parent,const char *name)
 
 //   resize(300,170);
 //   setCaption(i18n("Ai Setup"));
-   setHelp("kspaceduel/ai.html","");
+   setHelp( "Ai" );
    
    QHBox* box = makeHBoxMainWidget();
    
@@ -619,7 +620,7 @@ HitpointSetup::HitpointSetup(SOptions *opt,QWidget *parent,const char *name)
    options=opt;
 
 //   setCaption(i18n("Handicap Setup"));
-   setHelp( "kspaceduel/options.html", "OptionsHandicap" );
+   setHelp( "OptionsHandicap" );
 
    QSlider *blueSlider,*redSlider;
    QGroupBox *box=new QGroupBox(i18n("Hit points"),this);
@@ -684,7 +685,7 @@ GraphicSetup::GraphicSetup(SOptions *opt,QWidget *parent,const char *name)
    QGroupBox *box=new QGroupBox(i18n("Graphics"),this);
    setMainWidget( box );
 
-   setHelp( "kspaceduel/options.html", "OptionsGraphics" );
+   setHelp( "OptionsGraphics" );
 
    QLabel *refreshlabel=new QLabel(i18n("Refresh time"),box);
    refreshslider=new QSlider(10,100,10,refreshtime,QSlider::Horizontal,box);
@@ -720,3 +721,5 @@ void GraphicSetup::slotDefault()
    refreshslider->setValue(refreshtime);
    refreshnumber->display(refreshtime);
 }
+
+#include "dialogs.moc"
